import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
var _excluded = ["getContainer", "motion", "prefixCls", "maxCount", "top", "bottom"];
import * as React from 'react';
import Notifications from './Notifications';

var defaultGetContainer = function defaultGetContainer() {
  return document.body;
};

export default function useNotification() {
  var rootConfig = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _rootConfig$getContai = rootConfig.getContainer,
      getContainer = _rootConfig$getContai === void 0 ? defaultGetContainer : _rootConfig$getContai,
      motion = rootConfig.motion,
      prefixCls = rootConfig.prefixCls,
      maxCount = rootConfig.maxCount,
      top = rootConfig.top,
      bottom = rootConfig.bottom,
      shareConfig = _objectWithoutProperties(rootConfig, _excluded);

  var _React$useState = React.useState(),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      container = _React$useState2[0],
      setContainer = _React$useState2[1];

  var notificationsRef = React.useRef();
  var contextHolder = /*#__PURE__*/React.createElement(Notifications, {
    container: container,
    ref: notificationsRef,
    prefixCls: prefixCls,
    motion: motion,
    maxCount: maxCount,
    top: top,
    bottom: bottom
  });

  var _React$useState3 = React.useState([]),
      _React$useState4 = _slicedToArray(_React$useState3, 2),
      taskQueue = _React$useState4[0],
      setTaskQueue = _React$useState4[1]; // ========================= Refs =========================


  var api = React.useMemo(function () {
    return {
      open: function open(config) {
        var _config$key;

        var mergedConfig = _objectSpread(_objectSpread(_objectSpread({}, shareConfig), config), {}, {
          key: (_config$key = config.key) !== null && _config$key !== void 0 ? _config$key : Date.now()
        });

        setTaskQueue(function (queue) {
          return [].concat(_toConsumableArray(queue), [{
            type: 'open',
            config: mergedConfig
          }]);
        });
      },
      close: function close(key) {
        setTaskQueue(function (queue) {
          return [].concat(_toConsumableArray(queue), [{
            type: 'close',
            key: key
          }]);
        });
      },
      destroy: function destroy() {
        setTaskQueue(function (queue) {
          return [].concat(_toConsumableArray(queue), [{
            type: 'destroy'
          }]);
        });
      }
    };
  }, []); // ======================= Container ======================
  // React 18 should all in effect that we will check container in each render
  // Which means getContainer should be stable.

  React.useEffect(function () {
    setContainer(getContainer());
  }); // ======================== Effect ========================

  React.useEffect(function () {
    // Flush task when node ready
    if (notificationsRef.current && taskQueue.length) {
      taskQueue.forEach(function (task) {
        switch (task.type) {
          case 'open':
            notificationsRef.current.open(task.config);
            break;

          case 'close':
            notificationsRef.current.close(task.key);
            break;

          case 'destroy':
            notificationsRef.current.destroy();
            break;
        }
      });
      setTaskQueue([]);
    }
  }, [taskQueue]); // ======================== Return ========================

  return [api, contextHolder];
}